-- ===========================================================================
-- Botwatcher 1.0.3 - Plugin for Vendetta Online (all platforms)
-- ===========================================================================
-- Created by Snib in June 2022
--
-- Automatically adds respawning bots found in unmonitored sectors at the top
-- of your system notes for that sector. The format is the same as provided by
-- the game for monitored and guarded sectors, but the typically more 
-- dangerous bots classes are sorted first.
--
-- This is done at the time you spawn in a sector. If you enter the sector
-- when all of a type of bot has just been killed and not respawned yet, then
-- then this type will not be recorded. Just visit the sector again later and
-- it will be corrected. The same goes for entering the sector out of radar
-- range of the bots. Bots will not be recorded during ion storms.
--
-- Only bots permanently found in a sector are recorded. The plugin tries to
-- exclude unrats, voys, mission bots, hive infestations and similar
-- "visitors" to a sector.
--
-- This plugin has no commands or configuration options.
-- ===========================================================================

-- the global table is the API, other plugins can use this
declare("Botwatcher", {})
Botwatcher = {
    prefix = "Bots Sighted: ",
    sort_order = {
        Overseer = 1,
        Assault = 2,
        Guardian = 3,
        Collector = 4,
        Transport = 5,
        Observer = 6,
    },
    -- returns bot type based on sort_order above
    GetBotType = function(name)
        local result
        for type in pairs(Botwatcher.sort_order) do
            result = name:match(type)
            if result then break end
        end
        return result
    end,
    -- returns the entire "Bots Sighted: " string, or nil
    GetBotsSighted = function(sectorId)
        if sectorId then
            local bots_sighted = GetBotSightedInfoForSector(sectorId)
            if bots_sighted == "\n" then
                local systemId = GetSystemID(sectorId)
                if SystemNotes[systemId] then
                    local notes = SystemNotes[systemId][sectorId]
                    if notes then
                        return notes:match("(" .. Botwatcher.prefix .. ".-\n)")
                    end
                end
            else
                return bots_sighted
            end
        end
    end,
    -- returns a table with the bot names
    GetBotsSightedList = function(sectorId)
        local bots = {}
        if sectorId then
            local bots_sighted = Botwatcher.GetBotsSighted(sectorId)
            if bots_sighted then
                for bot in bots_sighted:sub(14):gmatch(" (.-)[,\n]") do
                    table.insert(bots, bot)
                end
            end
        end
        if next(bots) then return bots end
    end
}

-- now the plugin proper

local function botwatch()
    local sectorId = GetCurrentSectorid()
    -- only watch unmonitored sectors and skip ion storms 
    -- XXX: Technically we should also skip fogged sectors but I do not think
    -- any of them have bots, anyway
    if GetSectorMonitoredStatus() ~= 1 or IsStormPresent() or
        GetBotSightedInfoForSector(sectorId) ~= "\n" then return end

    local bots = {}
    local isQueen = false
    ForEachPlayer(function(charid)
        if isQueen then return end
        -- exception for wormhole sector in unknown system
        if (charid > 1000000000 or sectorId == 8200) and IsNPC(charid) then
            local name = GetPlayerName(charid)
            if name:find("Leviathan") ~= nil or name:find("Queen") ~= nil then
                -- skip queen/levi sectors, ids don't help us distinguish here
                isQueen = true
                return
            end
            local bot_type = Botwatcher.GetBotType(name)
            if bot_type then
                name = name:match("*(.+) .-")
                if name then bots[name] = bot_type end
            end
        end
    end)
    if isQueen then return end
    local text = ""
    local sorted_bots = {}
    for bot, type in pairs(bots) do
        table.insert(sorted_bots, { bot, Botwatcher.sort_order[type] })
    end
    table.sort(sorted_bots, function(a, b)
        local idx = (a[2] == b[2]) and 1 or 2
        return a[idx] < b[idx]
    end)
    for _, bot in ipairs(sorted_bots) do
        text = text .. bot[1] .. ", "
    end

    local systemId = GetCurrentSystemid()
    local notes = ""
    if SystemNotes[systemId] and SystemNotes[systemId][sectorId] then
        notes = SystemNotes[systemId][sectorId]
    end
    if text ~= "" then text = Botwatcher.prefix .. text:sub(1, -3) .. "\n" end
    notes = text .. notes:gsub(Botwatcher.prefix .. ".-\n", "")
    if notes == "" then
        notes = nil
    elseif not SystemNotes[systemId] then
        SystemNotes[systemId] = { colors = {} }
    end
    if notes ~= SystemNotes[systemId][sectorId] then
        SystemNotes[systemId][sectorId] = notes
        SystemNotes[systemId].colors[sectorId] = SystemNotes[systemId].colors[sectorId] or 255
        SaveSystemNotes(spickle(SystemNotes[systemId]), systemId)
    end
end

RegisterEvent(botwatch, "SHIP_SPAWN_CINEMATIC_FINISHED")
